@@ -9,7 +9,9 @@ UDP servers and clients.
 
 import errno
 import socket
+from typing import Union
 
+from thor.dns import lookup
 from thor.loop import EventSource, LoopBase
 
 
@@ -25,24 +27,27 @@ class UdpEndpoint(EventSource):
     > s = UdpEndpoint(host, port)
     > s.on('datagram', datagram_handler)
     """
+
     recv_buffer = 8192
     _block_errs = set([errno.EAGAIN, errno.EWOULDBLOCK])
 
     def __init__(self, loop: LoopBase = None) -> None:
         EventSource.__init__(self, loop)
+        self.host = None  # type: bytes
+        self.port = None  # type: int
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         self.sock.setblocking(False)
-        self.max_dgram = min((2**16 - 40), self.sock.getsockopt(
-            socket.SOL_SOCKET, socket.SO_SNDBUF
-        ))
-        self.on('fd_readable', self.handle_datagram)
+        self.max_dgram = min(
+            (2 ** 16 - 40), self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
+        )
+        self.on("fd_readable", self.handle_datagram)
         self.register_fd(self.sock.fileno())
 
     def __repr__(self) -> str:
         status = [self.__class__.__module__ + "." + self.__class__.__name__]
         return "<%s at %#x>" % (", ".join(status), id(self))
 
-    def bind(self, host: str, port: int) -> None:
+    def bind(self, host: bytes, port: int) -> None:
         """
         Bind the socket bound to host:port. If called, must be before
         sending or receiving.
@@ -50,21 +55,26 @@ class UdpEndpoint(EventSource):
         Can raise socket.error if binding fails.
         """
         # TODO: IPV6
+        self.host = host
+        self.port = port
         self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self.sock.bind((host, port))
+        lookup(host, self._continue_bind)
+
+    def _continue_bind(self, dns_result: Union[str, Exception]) -> None:
+        self.sock.bind((dns_result, self.port))
 
     def shutdown(self) -> None:
         "Close the listening socket."
-        self.removeListeners('fd_readable')
+        self.removeListeners("fd_readable")
         self.sock.close()
         # TODO: emit close?
 
     def pause(self, paused: bool) -> None:
         "Control incoming datagram events."
         if paused:
-            self.event_del('fd_readable')
+            self.event_del("fd_readable")
         else:
-            self.event_add('fd_readable')
+            self.event_add("fd_readable")
 
     def send(self, datagram: bytes, host: str, port: int) -> None:
         "send datagram to host:port."
@@ -72,7 +82,7 @@ class UdpEndpoint(EventSource):
             self.sock.sendto(datagram, (host, port))
         except socket.error as why:
             if why in self._block_errs:
-                pass # we drop these on the floor. It's UDP, after all.
+                pass  # we drop these on the floor. It's UDP, after all.
             else:
                 raise
 
@@ -88,4 +98,4 @@ class UdpEndpoint(EventSource):
                     break
                 else:
                     raise
-            self.emit('datagram', data, addr[0], addr[1])
+            self.emit("datagram", data, addr[0], addr[1])
